@@ -47,12 +47,11 @@
 #include "vidmodeproc.h"
 #include "xf86cmap.h"
 
-static int VidModeGeneration = 0;
-static int VidModeIndex = -1;
+static DevPrivateKey VidModeKey = NULL;
 static int VidModeCount = 0;
 static Bool VidModeClose(int i, ScreenPtr pScreen);
 
-#define VMPTR(p) ((VidModePtr)(p)->devPrivates[VidModeIndex].ptr)
+#define VMPTR(p) ((VidModePtr)dixLookupPrivate(&(p)->devPrivates, VidModeKey))
 
 #endif
 
@@ -68,22 +67,15 @@ VidModeExtensionInit(ScreenPtr pScreen)
 #ifdef XF86VIDMODE
     VidModePtr pVidMode;
     
-    DEBUG_P("VidModeExtensionInit");
-
     if (!xf86GetVidModeEnabled()) {
 	DEBUG_P("!xf86GetVidModeEnabled()");
 	return FALSE;
     }
 
-    if (serverGeneration != VidModeGeneration) {
-	if ((VidModeIndex = AllocateScreenPrivateIndex()) < 0) {
-	    DEBUG_P("AllocateScreenPrivateIndex() failed");
-	    return FALSE;
-	}
-	VidModeGeneration = serverGeneration;
-    }
+    VidModeKey = &VidModeKey;
 
-    if (!(pScreen->devPrivates[VidModeIndex].ptr = xcalloc(sizeof(VidModeRec), 1))) {
+    if (!dixSetPrivate(&pScreen->devPrivates, VidModeKey,
+		       xcalloc(sizeof(VidModeRec), 1))) {
 	DEBUG_P("xcalloc failed");
 	return FALSE;
     }
@@ -109,8 +101,6 @@ VidModeClose(int i, ScreenPtr pScreen)
 {
     VidModePtr pVidMode = VMPTR(pScreen);
 
-    DEBUG_P("VidModeClose");
-
     /* This shouldn't happen */
     if (!pVidMode)
 	return FALSE;
@@ -118,10 +108,9 @@ VidModeClose(int i, ScreenPtr pScreen)
     pScreen->CloseScreen = pVidMode->CloseScreen;
 
     if (--VidModeCount == 0) {
-	if (pScreen->devPrivates[VidModeIndex].ptr)
-	  xfree(pScreen->devPrivates[VidModeIndex].ptr);
-	pScreen->devPrivates[VidModeIndex].ptr = NULL;
-	VidModeIndex = -1;
+	xfree(dixLookupPrivate(&pScreen->devPrivates, VidModeKey));
+	dixSetPrivate(&pScreen->devPrivates, VidModeKey, NULL);
+	VidModeKey = NULL;
     }
     return pScreen->CloseScreen(i, pScreen);
 }
@@ -132,10 +121,8 @@ VidModeAvailable(int scrnIndex)
     ScrnInfoPtr pScrn;
     VidModePtr pVidMode;
 
-    DEBUG_P("VidModeAvailable");
-
-    if (VidModeIndex < 0) {
-	DEBUG_P("VidModeIndex < 0");
+    if (VidModeKey == NULL) {
+	DEBUG_P("VidModeKey == NULL");
 	return FALSE;
     }
  
@@ -159,8 +146,6 @@ VidModeGetCurrentModeline(int scrnIndex, pointer *mode, int *dotClock)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetCurrentModeline");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -176,8 +161,6 @@ VidModeGetDotClock(int scrnIndex, int Clock)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetDotClock");
-
     if (!VidModeAvailable(scrnIndex))
 	return 0;
 
@@ -193,8 +176,6 @@ VidModeGetNumOfClocks(int scrnIndex, Bool *progClock)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetNumOfClocks");
-
     if (!VidModeAvailable(scrnIndex))
 	return 0;
 
@@ -214,8 +195,6 @@ VidModeGetClocks(int scrnIndex, int *Clocks)
     ScrnInfoPtr pScrn;
     int i;
 
-    DEBUG_P("VidModeGetClocks");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -237,8 +216,6 @@ VidModeGetFirstModeline(int scrnIndex, pointer *mode, int *dotClock)
     ScrnInfoPtr pScrn;
     VidModePtr pVidMode;
 
-    DEBUG_P("VidModeGetFirstModeline");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -263,8 +240,6 @@ VidModeGetNextModeline(int scrnIndex, pointer *mode, int *dotClock)
     VidModePtr pVidMode;
     DisplayModePtr p;
 
-    DEBUG_P("VidModeGetNextModeline");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -288,8 +263,6 @@ VidModeDeleteModeline(int scrnIndex, pointer mode)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeDeleteModeline");
-
     if ((mode == NULL) || (!VidModeAvailable(scrnIndex)))
 	return FALSE;
 
@@ -303,8 +276,6 @@ VidModeZoomViewport(int scrnIndex, int zoom)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeZoomViewPort");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -318,8 +289,6 @@ VidModeSetViewPort(int scrnIndex, int x, int y)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeSetViewPort");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -341,8 +310,6 @@ VidModeGetViewPort(int scrnIndex, int *x, int *y)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetViewPort");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -359,8 +326,6 @@ VidModeSwitchMode(int scrnIndex, pointer mode)
     DisplayModePtr pTmpMode;
     Bool retval;
 
-    DEBUG_P("VidModeSwitchMode");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -381,8 +346,6 @@ VidModeLockZoom(int scrnIndex, Bool lock)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeLockZoom");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -400,8 +363,6 @@ VidModeGetMonitor(int scrnIndex, pointer *monitor)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetMonitor");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -416,8 +377,6 @@ VidModeCheckModeForMonitor(int scrnIndex, pointer mode)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeCheckModeForMonitor");
-
     if ((mode == NULL) || (!VidModeAvailable(scrnIndex)))
 	return MODE_ERROR;
 
@@ -431,8 +390,6 @@ VidModeCheckModeForDriver(int scrnIndex, pointer mode)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeCheckModeForDriver");
-
     if ((mode == NULL) || (!VidModeAvailable(scrnIndex)))
 	return MODE_ERROR;
 
@@ -447,8 +404,6 @@ VidModeSetCrtcForMode(int scrnIndex, pointer mode)
     ScrnInfoPtr pScrn;
     DisplayModePtr ScreenModes;
     
-    DEBUG_P("VidModeSetCrtcForMode");
-
     if ((mode == NULL) || (!VidModeAvailable(scrnIndex)))
 	return;
 
@@ -467,8 +422,6 @@ VidModeAddModeline(int scrnIndex, pointer mode)
 {
     ScrnInfoPtr pScrn;
     
-    DEBUG_P("VidModeAddModeline");
-
     if ((mode == NULL) || (!VidModeAvailable(scrnIndex)))
 	return FALSE;
 
@@ -491,8 +444,6 @@ VidModeGetNumOfModes(int scrnIndex)
     pointer mode = NULL;
     int dotClock= 0, nummodes = 0;
   
-    DEBUG_P("VidModeGetNumOfModes");
-
     if (!VidModeGetFirstModeline(scrnIndex, &mode, &dotClock))
 	return nummodes;
 
@@ -509,8 +460,6 @@ VidModeSetGamma(int scrnIndex, float red, float green, float blue)
     ScrnInfoPtr pScrn;
     Gamma gamma;
 
-    DEBUG_P("VidModeSetGamma");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
@@ -529,8 +478,6 @@ VidModeGetGamma(int scrnIndex, float *red, float *green, float *blue)
 {
     ScrnInfoPtr pScrn;
 
-    DEBUG_P("VidModeGetGamma");
-
     if (!VidModeAvailable(scrnIndex))
 	return FALSE;
 
